/**
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $id.java
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.engine.kpi;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import javax.jbi.JBIException;

import org.ow2.petals.component.framework.se.AbstractServiceEngine;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.engine.kpi.listener.NotificationStorage;
import org.ow2.petals.engine.kpi.su.KpiServiceUnitManager;

import com.ebmwebsourcing.petalsview.service.notification.Notification;

/**
 * 
 * @author tdejean - eBM WebSourcing -
 * 
 */
public class ServiceEngine extends AbstractServiceEngine {

    private KpiServiceUnitManager kpiServiceUnitManager = null;

    private BlockingQueue<Notification> notificationBlockingQueue = null;

    private NotificationStorage notificationStorage = null;

    private Thread notificationStorageThread = null;

    @Override
    protected AbstractServiceUnitManager createServiceUnitManager() {
        this.kpiServiceUnitManager = new KpiServiceUnitManager(this);
        return this.kpiServiceUnitManager;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.AbstractComponent#doInit()
     */
    @Override
    protected void doInit() throws JBIException {
        this.kpiServiceUnitManager.createKeyProcessIndicatorService();
        this.notificationBlockingQueue = new LinkedBlockingQueue<Notification>();
        try {
            final String bundleSizeString = this.getComponentExtensions().get("bundle_size");
            if (bundleSizeString != null) {
                final int bundleSize = Integer.parseInt(bundleSizeString);
                this.notificationStorage = new NotificationStorage(this.notificationBlockingQueue,
                        this.getLogger(), bundleSize);
            } else {
                this.notificationStorage = new NotificationStorage(this.notificationBlockingQueue,
                        this.getLogger());
            }
        } catch (final Exception e) {
            throw new JBIException(e);
        }
    }

    /**
     * Get the notificationBlockingQueue
     * 
     * @return the notificationBlockingQueue
     */
    public BlockingQueue<Notification> getNotificationBlockingQueue() {
        return this.notificationBlockingQueue;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.AbstractComponent#doStart()
     */
    @Override
    protected void doStart() throws JBIException {
        this.notificationStorage.setRun(true);
        this.notificationStorageThread = new Thread(this.notificationStorage);
        this.notificationStorageThread.start();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.AbstractComponent#doStop()
     */
    @Override
    protected void doStop() throws JBIException {
        this.notificationStorage.setRun(false);
        this.notificationStorageThread.interrupt();
        this.notificationStorageThread = null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.AbstractComponent#doShutdown()
     */
    @Override
    protected void doShutdown() throws JBIException {
        this.kpiServiceUnitManager = null;
        this.notificationBlockingQueue = null;
        this.notificationStorage = null;
    }
}
